/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

import java.lang.reflect.Constructor;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.annotation.ConfigurationTarget;
import org.t2framework.commons.exception.NoConfigurationTargetAnnotationException;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.ConfigContainer;
import org.t2framework.commons.meta.ConstructorDesc;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.meta.impl.ConstructorDescImpl;
import org.t2framework.commons.module.Modules;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Reflections.ConstructorUtil;
import org.t2framework.lucy.behavior.ComponentBehavior;
import org.t2framework.lucy.behavior.DestroyMethodBehavior;
import org.t2framework.lucy.behavior.InitMethodBehavior;
import org.t2framework.lucy.behavior.InjectBehavior;
import org.t2framework.lucy.behavior.PropertiesInjectBehavior;
import org.t2framework.lucy.behavior.PrototypeBehavior;
import org.t2framework.lucy.behavior.SingletonBehavior;
import org.t2framework.lucy.config.meta.InstanceConfig;
import org.t2framework.lucy.config.meta.InstanceConfigFactory;
import org.t2framework.lucy.exception.LucyStartupFailException;
import org.t2framework.lucy.module.LucyAopModule;
import org.t2framework.lucy.spi.Behavior;
import org.t2framework.lucy.spi.LucyConfiguration;

/**
 * {@link ConfigBinderFactory} is a factory class of {@link ConfigBinder}.
 * {@link ConfigBinder} is a singleton class for the application.
 * 
 * @author shot
 * 
 */
public class ConfigBinderFactory {

	/**
	 * Default {@link ConfigBinder}.
	 */
	private static ConfigBinder binder;

	/**
	 * Set customized {@link ConfigBinder}.If setting {@link ConfigBinder}, it
	 * effects entire application.
	 * 
	 * @param binder
	 */
	public static synchronized void setConfigBinder(ConfigBinder binder) {
		ConfigBinderFactory.binder = Assertion.notNull(binder);
	}

	/**
	 * Get {@link ConfigBinder}.
	 * 
	 * @return
	 */
	public static ConfigBinder getConfigBinder() {
		if (binder == null) {
			throw new LucyStartupFailException();
		}
		return binder;
	}

	/**
	 * Get {@link ConfigBinder} with {@link LucyConfiguration}.
	 * 
	 * @param conf
	 * @return
	 */
	public static ConfigBinder getConfigBinder(LucyConfiguration conf) {
		Assertion.notNull(conf);
		return getConfigBinder().loadConfiguration(conf);
	}

	/**
	 * Add {@link Behavior} to {@link ConfigBinder}.
	 * 
	 * @param b
	 */
	public static void addBehavior(Class<? extends Behavior> b) {
		Assertion.notNull(b);
		getConfigBinder().addBehavior(b);
	}

	/**
	 * Default {@link ConfigBinder} implementation.
	 * 
	 * @author shot
	 * 
	 */
	public static class DefaultConfigBinderImpl implements ConfigBinder {

		protected Map<Class<?>, Class<? extends Behavior>> customBehaviorMap = CollectionsUtil
				.newHashMap();

		protected Map<Class<?>, Class<? extends Behavior>> defaultBehaviorMap = CollectionsUtil
				.newHashMap();

		protected Map<Class<? extends Behavior>, ConstructorDesc<Behavior>> behaviorConstructorCache = CollectionsUtil
				.newHashMap();

		protected LucyConfiguration conf;

		public DefaultConfigBinderImpl() {
			init();
		}

		protected void init() {
			addDefaultBehavior(ComponentBehavior.class);
			addDefaultBehavior(InjectBehavior.class);
			addDefaultBehavior(SingletonBehavior.class);
			addDefaultBehavior(PrototypeBehavior.class);
			addDefaultBehavior(InitMethodBehavior.class);
			addDefaultBehavior(DestroyMethodBehavior.class);
			addDefaultBehavior(PropertiesInjectBehavior.class);
		}

		protected void addDefaultBehavior(
				Class<? extends Behavior> behaviorClass) {
			addBehavior(behaviorClass, this.defaultBehaviorMap);
		}

		protected void addBehavior(
				final Class<? extends Behavior> behaviorClass,
				final Map<Class<?>, Class<? extends Behavior>> map) {
			handleConfigurationTarget(behaviorClass,
					new ConfigurationTargetHandler<Object>() {

						@Override
						public Object handle(Class<?> targetClass,
								Class<? extends Behavior> behaviClass) {
							map.put(targetClass, behaviorClass);
							return null;
						}

					});
		}

		private static <R> R handleConfigurationTarget(
				Class<? extends Behavior> behaviorClass,
				ConfigurationTargetHandler<R> handler) {
			ConfigurationTarget a = behaviorClass
					.getAnnotation(ConfigurationTarget.class);
			if (a == null) {
				throw new NoConfigurationTargetAnnotationException(
						behaviorClass);
			}
			for (Class<?> targetClass : a.value()) {
				R r = handler.handle(targetClass, behaviorClass);
				if (r != null) {
					return r;
				}
			}
			return null;
		}

		/**
		 * {@.en Add {@link Behavior} class. }
		 * 
		 * <br />
		 * 
		 * {@.ja {@link Behavior}を足します．}
		 */
		@Override
		public void addBehavior(Class<? extends Behavior> behavior) {
			Assertion.notNull(behavior);
			addBehavior(behavior, this.customBehaviorMap);
		}

		/**
		 * {@.en Bind {@link BeanDesc} to {@link Behavior}. }
		 * 
		 * <br />
		 * 
		 * {@.ja {@link BeanDesc}の各設定項目をみて、適切な{@link Behavior}
		 * をバインドします．バインド結果は、 {@link ConfigBindResult}として返します．}
		 */
		@Override
		public synchronized <T> ConfigBindResult bind(final BeanDesc<T> beanDesc) {
			Assertion.notNull(beanDesc);
			ConfigBindResult result = new ConfigBindResult(beanDesc);
			bind(new Binder<ClassDesc<T>>() {

				@Override
				public void bind(ClassDesc<T> container, ConfigBindResult result) {
					for (Config config : container) {
						ConfigContainer configContainer = config
								.getConfigContainer();
						if (configContainer == null) {
							configContainer = container;
						}
						Behavior b = createBehavior(beanDesc, config,
								configContainer);
						if (b != null) {
							result.addBehavior(container, b);
						}
					}
				}

			}, beanDesc.getClassDesc(), result);

			bind(new Binder<List<MethodDesc>>() {

				@Override
				public void bind(List<MethodDesc> list, ConfigBindResult result) {
					for (MethodDesc md : list) {
						for (Config config : md) {
							Behavior b = createBehavior(beanDesc, config, md);
							if (b != null) {
								result.addBehavior(md, b);
							}
						}
					}
				}

			}, beanDesc.getAllMethodDesc(), result);

			bind(new Binder<List<PropertyDesc<T>>>() {

				@Override
				public void bind(List<PropertyDesc<T>> list,
						ConfigBindResult result) {
					for (PropertyDesc<?> pd : list) {
						for (Config config : pd) {
							Behavior b = createBehavior(beanDesc, config, pd);
							if (b != null) {
								result.addBehavior(pd, b);
							}
						}
					}
				}

			}, beanDesc.getAllPropertyDesc(), result);
			return result;
		}

		private <T> void bind(Binder<T> binder, T container,
				ConfigBindResult result) {
			binder.bind(container, result);
		}

		protected <T> Behavior createBehavior(BeanDesc<T> beanDesc,
				Config config, ConfigContainer container) {
			final Class<?> type = config.getType();
			return createBehavior(beanDesc, type, config, container);
		}

		/**
		 * Creating behavior using constructors.The construcors' parameters must
		 * be a {@link BeanDesc} and a {@link Config}.
		 * 
		 * @param <T>
		 * @param beanDesc
		 * @param type
		 * @param config
		 * @param container
		 * @return
		 */
		protected <T> Behavior createBehavior(BeanDesc<T> beanDesc,
				Class<?> type, Config config, ConfigContainer container) {
			final Class<? extends Behavior> clazz = findBehaviorClass(type);
			if (clazz != null) {
				ConstructorDesc<Behavior> cd = this.behaviorConstructorCache
						.get(clazz);
				if (cd == null) {
					cd = new ConstructorDescImpl<Behavior>(clazz);
					this.behaviorConstructorCache.put(clazz, cd);
				}
				Constructor<Behavior> c = cd.getSuitableConstructor(beanDesc,
						container, config);
				final Object[] args = new Object[] { beanDesc, container,
						config };
				return ConstructorUtil.newInstance(c, args);
			} else {
				return null;
			}
		}

		protected Class<? extends Behavior> findBehaviorClass(
				final Class<?> type) {
			Class<? extends Behavior> clazz = this.customBehaviorMap.get(type);
			if (clazz != null) {
				return clazz;
			}
			LucyAopModule aopModule = Modules.getModule(LucyAopModule.class);
			if (aopModule != null) {
				Class<? extends Behavior> aspectBehaviorClass = aopModule
						.getAspectBehaviorClass();
				clazz = handleConfigurationTarget(
						aspectBehaviorClass,
						new ConfigurationTargetHandler<Class<? extends Behavior>>() {

							@Override
							public Class<? extends Behavior> handle(
									Class<?> targetClass,
									Class<? extends Behavior> behaviorClass) {
								if (targetClass == type) {
									return behaviorClass;
								}
								return null;
							}

						});
			}
			if (clazz != null) {
				return clazz;
			}
			return this.defaultBehaviorMap.get(type);
		}

		@Override
		public ConfigBinder loadConfiguration(final LucyConfiguration conf) {
			this.conf = Assertion.notNull(conf);
			return this;
		}

		@Override
		public void addInstanceConfig(InstanceConfig instanceConfig) {
			InstanceConfigFactory.addInstanceConfig(Assertion
					.notNull(instanceConfig));
		}

		private static interface Binder<T> {
			void bind(T container, ConfigBindResult result);
		}

		private static interface ConfigurationTargetHandler<R> {
			R handle(Class<?> targetClass,
					Class<? extends Behavior> behaviorClass);
		}
	}
}
