/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy;

import org.t2framework.commons.Disposer;
import org.t2framework.commons.util.Assertion;
import org.t2framework.lucy.exception.LucyStartupFailException;
import org.t2framework.lucy.impl.DefaultLucyConfiguration;
import org.t2framework.lucy.impl.LucyImpl;
import org.t2framework.lucy.spi.LucyConfiguration;

/**
 * 
 * {@.en Bootstrap class for Lucy.}
 * 
 * <br />
 * 
 * {@.ja {@code Lucy}の起動クラスです．Lucyの初期化、生成(
 * {@link #init(LucyConfiguration, Object)})、破棄({@link #destroy()})を行う事が出来ます．}
 * 
 * @author shot
 * 
 */
public final class LucyBootstrap {

	/**
	 * {@link Lucy} instance.
	 */
	private static Lucy lucy;

	/**
	 * initialized flag.
	 */
	protected static boolean initialized = false;

	/**
	 * Initialize {@link Lucy}.
	 * 
	 * @return
	 */
	public static Lucy init() {
		return init((String) null);
	}

	/**
	 * Initialize {@link Lucy} with component definition path like xml path.
	 * 
	 * @param componentDefinitionPath
	 * @return
	 */
	public static Lucy init(final String componentDefinitionPath) {
		return init(new DefaultLucyConfiguration(), componentDefinitionPath);
	}

	/**
	 * Initialize {@link Lucy} with {@link LucyConfiguration}.
	 * 
	 * @param config
	 * @return
	 */
	public static Lucy init(final LucyConfiguration config) {
		return init(config, null);
	}

	/**
	 * 
	 * {@.en Initialize {@link Lucy} with {@link LucyConfiguration} and outer
	 * component definition path. The configuration instance must not be null. }
	 * 
	 * <br />
	 * 
	 * {@.ja {@code Lucy}を初期化します.初期化時に{@code LucyConfiguration}
	 * とコンポーネントの定義を読み込み、{@code Lucy}を生成します.}
	 * 
	 * @param config
	 * @param componentDefinition
	 * @return
	 */
	public static synchronized Lucy init(final LucyConfiguration config,
			final Object componentDefinition) {
		Assertion.notNull(config);
		if (initialized) {
			return lucy;
		}
		if (lucy == null) {
			lucy = new LucyImpl(config);
		}
		lucy.init();
		if (componentDefinition != null) {
			lucy.load(componentDefinition);
		}
		return lucy;
	}

	/**
	 * 
	 * {@.en Get instantiated {@link Lucy}.If there is no {@link Lucy}
	 * instance, throw {@link LucyStartupFailException}. }
	 * 
	 * <br />
	 * 
	 * {@.ja {@code Lucy}インスタンスを返します.インスタンスが生成されていない場合、{@code
	 * LucyStartupFailException}が発生します.}
	 * 
	 * @return
	 */
	public static Lucy getLucy() {
		if (lucy == null) {
			throw new LucyStartupFailException();
		}
		return lucy;
	}

	/**
	 * 
	 * {@.en Destroy instantiated {@link Lucy} instance, and
	 * {@link Disposer#dispose()} resources. }
	 * 
	 * <br />
	 * 
	 * {@.ja {@code Lucy}を破棄します.Lucy自身の破棄後に{@link Disposer#dispose()}
	 * を呼び出します.}
	 * 
	 */
	public static synchronized void destroy() {
		if (lucy != null) {
			lucy.destroy();
			lucy = null;
		}
		Disposer.dispose();
		initialized = false;
	}

}
