/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.meta;

import org.t2framework.commons.meta.ConstructorDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.util.StringUtil;
import org.t2framework.lucy.config.meta.impl.ConstructorInjectConfig;
import org.t2framework.lucy.config.meta.impl.PropertyInjectConfig;

/**
 * 
 * {@.en {@link AutoInject} is the enum of auto injection type.}
 * 
 * <br />
 * 
 * {@.ja {@link AutoInject}は自動設定のための振る舞いを持つenumです．}
 * 
 * @author shot
 * 
 */
public enum AutoInject {

	/**
	 * In case of autoinject="property".
	 */
	PROPERTY() {

		@Override
		public String getName() {
			return "property";
		}

		@Override
		public <T> void setupAutoInject(ConfigurableBeanDesc<T> beanDesc) {
			for (PropertyDesc<?> pd : beanDesc.getAllPropertyDesc()) {
				if (pd.hasConfig(InjectConfig.class) == false) {
					final Class<?>[] paramTypes = pd.getWriteMethod()
							.getParameterTypes();
					pd.addConfig(new PropertyInjectConfig(paramTypes, true));
				}
			}
		}

	},

	/**
	 * In case of autoinject="constructor".
	 */
	CONSTRUCTOR() {

		@Override
		public String getName() {
			return "constructor";
		}

		@Override
		public <T> void setupAutoInject(ConfigurableBeanDesc<T> beanDesc) {
			ConstructorDesc<T> cd = beanDesc.getConstructorDesc();
			if (cd.hasConfig(InjectConfig.class) == false) {
				InjectConfig injectConfig = new ConstructorInjectConfig(true);
				cd.addConfig(injectConfig);
				beanDesc.setConstructorDesc(cd);
			}
		}

	},

	/**
	 * In case of autoinject="all".
	 */
	ALL() {
		@Override
		public String getName() {
			return "all";
		}

		@Override
		public <T> void setupAutoInject(ConfigurableBeanDesc<T> beanDesc) {
			CONSTRUCTOR.setupAutoInject(beanDesc);
			PROPERTY.setupAutoInject(beanDesc);
		}
	},

	/**
	 * In case of autoinject="none".
	 */
	NONE() {

		@Override
		public String getName() {
			return "";
		}

		@Override
		public <T> void setupAutoInject(ConfigurableBeanDesc<T> beanDesc) {
			// no op;
		}

	};

	public abstract String getName();

	public abstract <T> void setupAutoInject(ConfigurableBeanDesc<T> beanDesc);

	/**
	 * 
	 * {@.en }
	 * 
	 * <br />
	 * 
	 * {@.ja 名前から{@link AutoInject}のenumを探します．}
	 * 
	 * @param name
	 * @return
	 */
	public static AutoInject get(String name) {
		if (StringUtil.isEmpty(name)) {
			return NONE;
		}
		name = name.toLowerCase().trim();
		if (name.equals(CONSTRUCTOR.getName())) {
			return CONSTRUCTOR;
		} else if (name.equals(PROPERTY.getName())) {
			return PROPERTY;
		} else if (name.equals(ALL.getName())) {
			return ALL;
		}
		return NONE;
	}

	@Override
	public String toString() {
		return getName();
	}

}
