/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.config.stax;

import java.io.BufferedInputStream;
import java.net.URL;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.Characters;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.ResourceUtil;
import org.t2framework.commons.util.StAXUtil;
import org.t2framework.commons.util.StreamUtil;
import org.t2framework.commons.util.URLUtil;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.exception.TagHandlerNotFoundException;

/**
 * 
 * {@.en A xml parser using StAX.}
 * 
 * <br />
 * 
 * {@.ja StAXを使ったXMLパーサです．}
 * 
 * @author shot
 * 
 */
public class StAXParser {

	protected final XmlEventHandlerRule rule;

	protected final XMLInputFactory factory;

	protected final LucyEventFilter filter = new LucyEventFilter();

	public StAXParser(XmlEventHandlerRule rule) {
		this.rule = Assertion.notNull(rule);
		this.factory = XMLInputFactory.newInstance();
		factory.setXMLResolver(new LucyXmlResolver());
	}

	/**
	 * 
	 * {@.en }
	 * 
	 * <br />
	 * 
	 * {@.ja XMLをパースして、適切なハンドラを呼びます．}
	 * 
	 * @param lucy
	 * @param path
	 */
	public void parse(Lucy lucy, String path) {
		BufferedInputStream bis = getBufferedInputStream(path);
		XMLEventReaderDelegate reader = null;
		try {
			XMLEventReader org = StAXUtil.createXMLEventReader(factory, bis);
			XMLEventReader filteredReader = StAXUtil.createFilteredReader(
					factory, org, filter);
			reader = new XMLEventReaderDelegate(filteredReader);
			XmlEventContext context = new XmlEventContext(lucy);
			for (; reader.hasNext();) {
				XMLEvent event = reader.nextEvent();
				context.setCurrentEvent(event);
				if (event.isStartElement()) {
					StartElement se = event.asStartElement();
					Attributes attributes = new Attributes(se);
					String tagname = se.getName().getLocalPart();
					XmlEventHandler handler = getHandler(tagname);
					handler.start(context, attributes);
				} else if (event.isEndElement()) {
					EndElement ee = event.asEndElement();
					String tagname = ee.getName().getLocalPart();
					XmlEventHandler handler = getHandler(tagname);
					String body = context.popBody();
					handler.end(context, body);
				} else if (event.isCharacters()) {
					Characters c = event.asCharacters();
					context.pushBody(c.getData());
				}
			}
		} finally {
			if (reader != null) {
				reader.close();
			}
			if (bis != null) {
				StreamUtil.close(bis);
			}
		}

	}

	protected XmlEventHandler getHandler(String tagname) {
		XmlEventHandler handler = rule.getHandler(tagname);
		if (handler == null) {
			throw new TagHandlerNotFoundException(tagname);
		}
		return handler;
	}

	protected BufferedInputStream getBufferedInputStream(String path) {
		URL url = ResourceUtil.getResource(path);
		return new BufferedInputStream(URLUtil.openStream(url));
	}
}
