/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.impl;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.BeanDescFactory;
import org.t2framework.commons.meta.NameDesc;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.lucy.BeanDescBag;
import org.t2framework.lucy.BeanDescStrategy;
import org.t2framework.lucy.ConfigBindResult;
import org.t2framework.lucy.LucyContext;
import org.t2framework.lucy.spi.Behavior;

/**
 * Context class for Lucy environment.This one has map of class to
 * {@link BeanDesc}, named {@link BeanDesc} list, and binding result of
 * {@link BeanDesc} to {@link Behavior}.
 * 
 * @author shot
 * 
 */
public class LucyContextImpl implements LucyContext {

	protected Map<Class<?>, BeanDescBag<?>> beanDescBagMap = CollectionsUtil
			.newConcurrentHashMap();

	protected List<BeanDesc<?>> namedBeanDescList = CollectionsUtil
			.newArrayList();

	protected Map<BeanDesc<?>, ConfigBindResult> bindMap = CollectionsUtil
			.newConcurrentHashMap();

	protected BeanDescStrategy beanDescStrategy;

	public LucyContextImpl(BeanDescStrategy beanDescStrategy) {
		this.beanDescStrategy = beanDescStrategy;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.t2framework.lucy.LucyContext#getBeanDescBag(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	public <T> BeanDescBag<T> getBeanDescBag(Class<? extends T> key) {
		return (BeanDescBag<T>) beanDescBagMap.get(key);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.t2framework.lucy.LucyContext#putBeanDesc(java.lang.Class,
	 * org.t2framework.commons.meta.BeanDesc)
	 */
	@Override
	@SuppressWarnings("unchecked")
	public <T> void putBeanDesc(Class<?> clazz,
			final BeanDesc<? extends T> beanDesc) {
		BeanDescBag bag = beanDescBagMap.get(clazz);
		if (bag == null) {
			bag = new BeanDescBag(clazz, beanDescStrategy);
			beanDescBagMap.put(clazz, bag);
		}
		bag.add(beanDesc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.t2framework.lucy.LucyContext#destroy()
	 */
	public void destroy() {
		BeanDescFactory.clear(beanDescBagMap.keySet());
		beanDescBagMap.clear();
		namedBeanDescList.clear();
		bindMap.clear();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.t2framework.lucy.LucyContext#getBeanDescList(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	public <T> List<BeanDesc<T>> getBeanDescList(Class<? super T> key) {
		BeanDescBag beanDescBag = beanDescBagMap.get(key);
		if (beanDescBag != null) {
			return beanDescBag.getBeanDescList();
		} else {
			return Collections.emptyList();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.t2framework.lucy.LucyContext#getBeanDescByIdentifier(java.lang.Object
	 * )
	 */
	@SuppressWarnings("unchecked")
	public <T> BeanDesc<T> getBeanDescByIdentifier(Object identifier) {
		BeanDesc<?> beanDesc = null;
		if (namedBeanDescList.isEmpty() == false) {
			for (BeanDesc<?> bd : namedBeanDescList) {
				NameDesc<?> nameDesc = bd.getNameDesc();
				if (nameDesc.isIdentified(identifier)) {
					beanDesc = bd;
					break;
				}
			}
		}
		return (BeanDesc<T>) beanDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.t2framework.lucy.LucyContext#addNamedBeanDesc(org.t2framework.commons
	 * .meta.BeanDesc)
	 */
	public <T> void addNamedBeanDesc(BeanDesc<T> beanDesc) {
		if (namedBeanDescList.contains(beanDesc) == false) {
			namedBeanDescList.add(beanDesc);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.t2framework.lucy.LucyContext#getConfigBindResult(org.t2framework.
	 * commons.meta.BeanDesc)
	 */
	public <T> ConfigBindResult getConfigBindResult(
			BeanDesc<? extends T> beanDesc) {
		return bindMap.get(beanDesc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.t2framework.lucy.LucyContext#hasConfigBindResult(org.t2framework.
	 * commons.meta.BeanDesc)
	 */
	public <T> boolean hasConfigBindResult(BeanDesc<? extends T> beanDesc) {
		return bindMap.containsKey(beanDesc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.t2framework.lucy.LucyContext#putConfigBindResult(org.t2framework.
	 * commons.meta.BeanDesc, org.t2framework.lucy.ConfigBindResult)
	 */
	public <T> void putConfigBindResult(BeanDesc<? extends T> beanDesc,
			ConfigBindResult bindResult) {
		bindMap.put(beanDesc, bindResult);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.t2framework.lucy.LucyContext#iterator()
	 */
	@Override
	public Iterator<ConfigBindResult> iterator() {
		return bindMap.values().iterator();
	}

	@Override
	public List<BeanDesc<?>> getNamedBeanDescList() {
		return namedBeanDescList;
	}

	@Override
	public <T> ConfigBindResult removeConfigBindResult(
			BeanDesc<? extends T> beanDesc) {
		if (bindMap.containsKey(beanDesc)) {
			return bindMap.remove(beanDesc);
		} else {
			return null;
		}
	}

	@Override
	public <T> void removeBeanDesc(BeanDesc<? extends T> beanDesc) {
		for (Class<?> c : beanDesc.getClassDesc().getDependencyClassesList()) {
			this.beanDescBagMap.remove(c);
		}
	}

}
