/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.impl;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.t2framework.commons.annotation.core.Component;
import org.t2framework.commons.exception.NoSuchComponentException;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.BeanDescFactory;
import org.t2framework.commons.meta.NameDesc;
import org.t2framework.commons.meta.PackageDesc;
import org.t2framework.commons.meta.PackageDescHolder;
import org.t2framework.commons.meta.impl.NameDescImpl;
import org.t2framework.commons.meta.impl.PackageDescHolderImpl;
import org.t2framework.commons.meta.impl.SimpleBeanDescImpl;
import org.t2framework.commons.util.AnnotationUtil;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Logger;
import org.t2framework.commons.util.Reflections.PackageUtil;
import org.t2framework.lucy.BeanDescBag;
import org.t2framework.lucy.BeanDescStrategy;
import org.t2framework.lucy.ComponentLoader;
import org.t2framework.lucy.ConfigBindResult;
import org.t2framework.lucy.ConfigBinder;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.LucyComponentRegistrar;
import org.t2framework.lucy.LucyContext;
import org.t2framework.lucy.LucyUtil;
import org.t2framework.lucy.exception.CyclicReferenceRuntimeException;
import org.t2framework.lucy.exception.LucyComponentRegistrarNotFoundRuntimeException;
import org.t2framework.lucy.exception.NoComponentAnnotationException;
import org.t2framework.lucy.spi.ComponentBuilder;
import org.t2framework.lucy.spi.LucyConfiguration;

/**
 * 
 * {@.en {@link Lucy} implementation. }
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
public class LucyImpl implements Lucy {

	protected Lifecycle phase;

	protected List<PackageDesc> packageDescList = CollectionsUtil
			.newArrayList();

	protected ConfigBinder binder;

	protected LucyConfiguration lucyConf;

	protected BeanDescStrategy beanDescStrategy;

	private ThreadLocal<BuildingPath> buildingPaths = new ThreadLocal<BuildingPath>();

	protected static Logger log = Logger.getLogger(LucyImpl.class);

	protected PackageDescHolder delegate = new PackageDescHolderImpl();

	protected LucyContext context;

	public LucyImpl() {
		this(new DefaultLucyConfiguration());
	}

	public LucyImpl(final LucyConfiguration conf) {
		Assertion.notNull(conf);
		this.lucyConf = conf;
		setLifecycle(Lifecycle.CONTAINER_CREATED);
	}

	public void init() {
		this.binder = lucyConf.loadConfigBinder();
		this.beanDescStrategy = lucyConf.getBeanDescStrategy(this);
		this.context = new LucyContextImpl(this.beanDescStrategy);
		registerLucy();
		setLifecycle(Lifecycle.CONTAINER_CONFIGURED);
	}

	@SuppressWarnings("unchecked")
	protected void registerLucy() {
		final SimpleBeanDescImpl<Lucy> lucyDesc = new SimpleBeanDescImpl<Lucy>(
				this, Lucy.class);
		registerBeanDescByClasses(lucyDesc, Arrays.asList(Lucy.class,
				PackageDescHolder.class, ComponentLoader.class));
	}

	@SuppressWarnings("unchecked")
	public <T> BeanDesc<T> getBeanDesc(Object identifier) {
		Assertion.notNull(identifier);
		if (identifier instanceof Class) {
			return (BeanDesc<T>) getBeanDesc((Class<T>) identifier);
		}
		BeanDesc<T> beanDesc = getBeanDescByIdentifier(identifier);
		buildComponent(identifier, beanDesc);
		return beanDesc;
	}

	@SuppressWarnings("unchecked")
	public <T> T get(Object identifier) {
		if (identifier instanceof Class) {
			return (T) get((Class<T>) identifier);
		}
		final BeanDesc<?> beanDesc = getBeanDescByIdentifier(identifier);
		return (T) buildComponent(identifier, beanDesc);
	}

	protected <T> BeanDesc<T> getBeanDescByIdentifier(Object identifier) {
		return this.context.getBeanDescByIdentifier(identifier);
	}

	public <T> BeanDesc<T> getBeanDesc(Class<? super T> key) {
		final BeanDesc<T> beanDesc = getBeanDesc0(key);
		buildComponent(key, beanDesc);
		return beanDesc;
	}

	public <T> T get(Class<? super T> key) {
		final BeanDesc<T> beanDesc = getBeanDesc0(key);
		return buildComponent(key, beanDesc);
	}

	@SuppressWarnings("unchecked")
	protected <T> BeanDesc<T> getBeanDesc0(Class<? super T> key) {
		final BeanDescBag beanDescBag = this.context.getBeanDescBag(key);
		if (beanDescBag != null) {
			return beanDescBag.getBeanDesc();
		} else {
			return null;
		}
	}

	protected <T> T buildComponent(final Object key, BeanDesc<T> beanDesc) {
		if (beanDesc == null) {
			return beanDescStrategy.<T> handleNoBeanDesc(key);
		}
		BuildingPath buildingPath = getBuildingPath();
		if (buildingPath.contains(beanDesc)) {
			if (beanDesc.isSingleton()) {
				return beanDesc.getComponent();
			} else if (buildingPath.isSingletonPresent() == false) {
				throw new CyclicReferenceRuntimeException(key, beanDesc);
			}
		}
		buildingPath.add(beanDesc);
		try {
			final T t = beanDesc.getComponent();
			if (t != null) {
				if (LucyUtil.isSimpleType(t.getClass())
						|| beanDesc.isSingleton()) {
					return t;
				} else {
					bind(beanDesc);
				}
			}
			ConfigBindResult result = this.context
					.getConfigBindResult(beanDesc);
			if (result == null) {
				throw new NoSuchComponentException(key);
			}
			return createComponentBuilder(beanDesc).build(t, result);
		} finally {
			buildingPath.remove(beanDesc);
			if (buildingPath.isEmpty()) {
				removeBuildingPath();
			}
		}
	}

	BuildingPath getBuildingPath() {
		BuildingPath buildingPath = buildingPaths.get();
		if (buildingPath == null) {
			buildingPath = new BuildingPath();
			buildingPaths.set(buildingPath);
		}
		return buildingPath;
	}

	void removeBuildingPath() {
		buildingPaths.set(null);
	}

	protected <T> ComponentBuilder createComponentBuilder(BeanDesc<T> beanDesc) {
		return lucyConf.createBuilder(beanDesc, this);
	}

	public <T> Lucy register(Class<? extends T> componentClass) {
		Assertion.notNull(componentClass, "componentClass");
		assertComponentAnnotation(componentClass);
		if (BeanDescFactory.hasCache(componentClass)) {
			return this;
		}
		final BeanDesc<T> beanDesc = BeanDescFactory
				.getBeanDesc(componentClass);
		register(beanDesc);
		return this;
	}

	@Override
	public <T> Lucy register(Class<? extends T> componentClass,
			Object identifier) {
		Assertion.notNull(componentClass, "componentClass");
		Assertion.notNull(identifier, "identifier");
		assertComponentAnnotation(componentClass);
		final BeanDesc<T> beanDesc = BeanDescFactory
				.getBeanDesc(componentClass);
		register(beanDesc, identifier);
		return this;
	}

	@Override
	public <T> Lucy register(BeanDesc<? extends T> beanDesc, Object identifier) {
		Assertion.notNull(beanDesc, "beanDesc");
		Assertion.notNull(identifier, "identifier");
		final NameDesc<T> nameDesc = createNameDesc(identifier);
		registerWithNameDesc(beanDesc, nameDesc);
		return this;
	}

	@SuppressWarnings("unchecked")
	protected <T> Lucy registerWithNameDesc(BeanDesc<? extends T> beanDesc,
			NameDesc<T> nameDesc) {
		Assertion.notNull(beanDesc, "beanDesc");
		Assertion.notNull(nameDesc, "nameDesc");
		beanDesc.setNameDesc((NameDesc) nameDesc);
		register(beanDesc);
		// TODO logging
		// log.debug("register component :" +
		// beanDesc.getConcreteClass().getName());
		return this;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T> Lucy register(T t) {
		Assertion.notNull(t, "instance");
		Class<T> componentClass = (Class<T>) t.getClass();
		assertComponentAnnotation(componentClass);
		if (BeanDescFactory.hasCache(t)) {
			return this;
		}
		BeanDesc<T> beanDesc = BeanDescFactory.getBeanDesc(t);
		register(beanDesc);
		return this;
	}

	@Override
	public <T> Lucy register(T t, Object identifier) {
		Assertion.notNull(t, "instance");
		Assertion.notNull(identifier, "identifier");
		assertComponentAnnotation(t.getClass());
		final BeanDesc<T> beanDesc = BeanDescFactory.getBeanDesc(t);
		return register(beanDesc, identifier);
	}

	public <T> Lucy register(final BeanDesc<? extends T> beanDesc) {
		Assertion.notNull(beanDesc, "beanDesc");
		final List<Class<?>> dependencyClassesList = beanDesc.getClassDesc()
				.getDependencyClassesList();
		registerBeanDescByClasses(beanDesc, dependencyClassesList);
		bind(beanDesc);
		return this;
	}

	protected <T> void bind(final BeanDesc<? extends T> beanDesc) {
		if (this.context.hasConfigBindResult(beanDesc)) {
			return;
		}
		final ConfigBindResult bindResult = binder.bind(beanDesc);
		if (bindResult.isInitialized() == false) {
			bindResult.initializeBehaviors();
		}
		this.context.putConfigBindResult(beanDesc, bindResult);
	}

	protected <T> void registerBeanDescByClasses(
			final BeanDesc<? extends T> beanDesc,
			List<Class<?>> dependencyClassesList) {
		if (beanDesc.hasName()) {
			this.context.addNamedBeanDesc(beanDesc);
		}
		for (Class<?> c : dependencyClassesList) {
			this.context.putBeanDesc(c, beanDesc);
		}
	}

	public synchronized void destroy() {
		setLifecycle(Lifecycle.CONTAINER_DESTROY);
		for (ConfigBindResult result : this.context) {
			result.destroyBehaviors();
		}
		this.context.destroy();
	}

	protected static <T> void assertComponentAnnotation(
			Class<? extends T> componentClass) {
		if (PackageUtil.isJavaPackage(componentClass)) {
			return;
		}
		if (AnnotationUtil.hasAnnotationType(Component.class, componentClass
				.getAnnotations()) == false) {
			throw new NoComponentAnnotationException(componentClass);
		}
	}

	@SuppressWarnings("unchecked")
	public <T> BeanDesc<T>[] getBeanDescs(Class<? super T> key) {
		return (BeanDesc<T>[]) getBeanDescList(key).toArray(
				(T[]) Array.newInstance(key, 0));
	}

	@SuppressWarnings("unchecked")
	public <T> T[] getAll(Class<? super T> key) {
		final List<BeanDesc<T>> beanDescList = getBeanDescList(key);
		T[] components = (T[]) Array.newInstance(key, beanDescList.size());
		int idx = 0;
		for (BeanDesc<T> beanDesc : beanDescList) {
			components[idx++] = buildComponent(key, beanDesc);
		}
		return components;
	}

	protected <T> List<BeanDesc<T>> getBeanDescList(Class<? super T> key) {
		return this.context.getBeanDescList(key);
	}

	@SuppressWarnings("unchecked")
	protected <T> NameDesc<T> createNameDesc(Object identifier) {
		if (NameDesc.class.isInstance(identifier)) {
			return NameDesc.class.cast(identifier);
		}
		return new NameDescImpl<T>(identifier);
	}

	@Override
	public void load(Object componentDefinition) {
		Assertion.notNull(componentDefinition);
		LucyComponentRegistrar<Object> registrar = lucyConf
				.getAcceptableComponentRegistrar(componentDefinition);
		if (registrar != null) {
			registrar.register(this, componentDefinition);
		} else {
			throw new LucyComponentRegistrarNotFoundRuntimeException(
					componentDefinition);
		}
	}

	@Override
	public <T> boolean hasComponent(Class<T> key) {
		final List<BeanDesc<T>> beanDescList = getBeanDescList(key);
		return beanDescList.isEmpty() == false;
	}

	@Override
	public boolean hasComponent(Object identifier) {
		return getBeanDesc(Assertion.notNull(identifier)) != null;
	}

	@SuppressWarnings("unchecked")
	@Override
	public synchronized <T> T injectDependency(T t) {
		register(t);
		Class clazz = t.getClass();
		return (T) get(clazz);
	}

	@Override
	public void addPackageDesc(PackageDesc packageDesc) {
		delegate.addPackageDesc(packageDesc);
	}

	@Override
	public PackageDesc getPackageDesc(String regex) {
		return delegate.getPackageDesc(regex);
	}

	@Override
	public Iterator<PackageDesc> iterator() {
		return delegate.iterator();
	}

	@Override
	public Iterator<Class<?>> iteratorClasses(String regex) {
		return delegate.iteratorClasses(regex);
	}

	@Override
	public Iterator<PackageDesc> iteratorPackageDescs(String regex) {
		return delegate.iteratorPackageDescs(regex);
	}

	/**
	 * Building path information.
	 * 
	 * @author shot
	 * 
	 */
	protected static class BuildingPath {

		List<BeanDesc<?>> beanDescList = new ArrayList<BeanDesc<?>>();

		public void add(BeanDesc<?> beanDesc) {
			beanDescList.add(beanDesc);
		}

		public boolean isEmpty() {
			return beanDescList.isEmpty();
		}

		public void remove(BeanDesc<?> beanDesc) {
			beanDescList.remove(beanDescList.size() - 1);
		}

		public boolean isSingletonPresent() {
			for (BeanDesc<?> bd : beanDescList) {
				if (bd.isSingleton()) {
					return true;
				}
			}
			return false;
		}

		public boolean contains(BeanDesc<?> beanDesc) {
			for (BeanDesc<?> bd : beanDescList) {
				if (bd == beanDesc) {
					return true;
				}
			}
			return false;
		}
	}

	public Lifecycle getLifecycle() {
		return phase;
	}

	public void setLifecycle(Lifecycle phase) {
		this.phase = phase;
	}

	@Override
	public <T> void pretend(Class<? extends T> original,
			Class<? extends T> newone) {
		if (phase == Lifecycle.CONTAINER_CONFIGURED) {
			pretend0(original, BeanDescFactory.getBeanDesc(newone));
		}
	}

	@Override
	public <T> void pretend(Class<? extends T> original, T instance) {
		if (phase == Lifecycle.CONTAINER_CONFIGURED) {
			pretend0(original, BeanDescFactory.getBeanDesc(instance));
		}
	}

	@Override
	public <T> void pretend(Object identifier, Class<? extends T> newone) {
		if (phase == Lifecycle.CONTAINER_CONFIGURED) {
			BeanDesc<T> beanDesc = BeanDescFactory.getBeanDesc(newone);
			pretend0(identifier, beanDesc);
		}
	}

	@Override
	public <T> void pretend(Object identifier, T instance) {
		if (phase == Lifecycle.CONTAINER_CONFIGURED) {
			BeanDesc<T> beanDesc = BeanDescFactory.getBeanDesc(instance);
			pretend0(identifier, beanDesc);
		}
	}

	protected <T> void pretend0(Class<? extends T> original,
			BeanDesc<T> beanDesc) {
		BeanDescBag<T> bag = this.context.getBeanDescBag(original);
		if (bag == null) {
			return;
		}
		List<BeanDesc<T>> list = bag.getBeanDescList();
		if (list.isEmpty() == false) {
			final BeanDesc<T> oldBeanDesc = list.get(0);
			if (oldBeanDesc.hasName()) {
				this.context.getNamedBeanDescList().remove(oldBeanDesc);
			}
			this.context.removeBeanDesc(oldBeanDesc);
			this.context.removeConfigBindResult(oldBeanDesc);
			register(beanDesc);
		}
	}

	protected <T> void pretend0(Object identifier,
			BeanDesc<? extends T> beanDesc) {
		BeanDesc<Object> oldBeanDesc = this.context
				.getBeanDescByIdentifier(identifier);
		if (oldBeanDesc != null) {
			if (oldBeanDesc.hasName()) {
				this.context.getNamedBeanDescList().remove(oldBeanDesc);
			}
			this.context.removeBeanDesc(oldBeanDesc);
			this.context.removeConfigBindResult(oldBeanDesc);
			register(beanDesc, identifier);
		}
	}
}
